classdef ShapeViewerMainFrame < handle
%SHAPEVIEWERMAINFRAME  Manages a figures that displays a document
%
%   output = ShapeViewerMainFrame(input)
%
%   Example
%   ShapeViewerMainFrame
%
%   See also
%
%
% ------
% Author: David Legland
% e-mail: david.legland@grignon.inra.fr
% Created: 2011-12-11,    using Matlab 7.9.0.529 (R2009b)
% Copyright 2011 INRA - Cepia Software Platform.


properties
    % reference to the main GUI instance
    gui;
   
    % list of handles to the various gui items
    handles;
    
    % the image document
    doc;
    
    % the set of mouse listeners.
    % Stored as an array of svui.app.Shape instances
    mouseListeners = [];
    
    % the currently selected tool
    currentTool = [];
    
    % the set of selected shapes, stored as a cell array
    selectedShapes = [];
    
    % remember where files were loaded and saved
    lastOpenPath = '.';
    lastSavePath = '.';
    
end

methods
    function this = ShapeViewerMainFrame(gui, doc)
        this.gui = gui;
        this.doc = doc;
        
        % create default figure
        fig = figure(...
            'MenuBar', 'none', ...
            'NumberTitle', 'off', ...
            'NextPlot', 'new', ...
            'Name', 'Shape Viewer', ...
            'CloseRequestFcn', @this.close, ...
            'ResizeFcn', @this.onFigureResized);
        
        % create main figure menu
        setupMenu(fig);
        setupLayout(fig);
        
        this.handles.figure = fig;
        
        updateDisplay(this);
        updateTitle(this);
        
        % setup listeners associated to the figure
        set(fig, 'WindowButtonDownFcn',     @this.processMouseButtonPressed);
        set(fig, 'WindowButtonUpFcn',       @this.processMouseButtonReleased);
        set(fig, 'WindowButtonMotionFcn',   @this.processMouseMoved);

        % setup mouse listener for display of mouse coordinates
        tool = svui.gui.tools.ShowCursorPositionTool(this, 'showMousePosition');
        addMouseListener(this, tool);
        
        tool = svui.gui.tools.SelectionTool(this, 'selection');
        addMouseListener(this, tool);
        this.currentTool = tool;
        
        
        set(fig, 'UserData', this);
        
        function setupMenu(hf)
            
            import svui.gui.actions.*;
            import svui.gui.tools.*;
            
            % File Menu Definition 
            
            fileMenu = uimenu(hf, 'Label', '&Files');
            
            addMenuItem(fileMenu, CreateNewDocAction(this), '&New Document');
           
            action = OpenPointsInTableAction(this, 'openPointSetInTable');
            addMenuItem(fileMenu, action, 'Import Point &Set', true);
            action = OpenPolygonInTableAction(this, 'openPolygonInTable');
            addMenuItem(fileMenu, action, 'Import Polygon');
            action = CloseCurrentDocAction(this, 'closeDoc');
            addMenuItem(fileMenu, action, '&Close', true);

            
            % Edit Menu Definition 
            
            editMenu = uimenu(hf, 'Label', '&Edit');
            
            addMenuItem(editMenu, SelectAllShapesAction(this),      'Select &All');
            addMenuItem(editMenu, DeleteSelectedShapesAction(this), '&Delete');
           
            addMenuItem(editMenu, SetSelectedShapeStyleAction(this),  'Set Display Style...', true);
            addMenuItem(editMenu, RenameSelectedShapeAction(this),  '&Rename', true);
           
            addMenuItem(editMenu, ZoomInAction(this), 'Zoom &In', true);
            addMenuItem(editMenu, ZoomOutAction(this), 'Zoom &Out');
            
            
             % Document Menu Definition 
            
            docMenu = uimenu(hf, 'Label', '&Document');
            addMenuItem(docMenu, AddNewDemoShapeAction(this), 'Add Demo Shape');
            addMenuItem(docMenu, AddPaperHenShapeAction(this), 'Add Paper Hen Shape');
            addMenuItem(docMenu, AddRandomPointsAction(this), 'Add Random Points');
            addMenuItem(docMenu, DisplaySelectionInfoAction(this), 'Display &Info', true);
            addMenuItem(docMenu, SetDocumentViewBoxAction(this), 'Set &View Box...', true);
            addMenuItem(docMenu, ToggleDocumentShowAxisLinesAction(this), 'Toggle &Axis Lines');
            addMenuItem(docMenu, ChangeUserUnitAction(this), 'Change &Unit...');
            addMenuItem(docMenu, RenameCurrentDocAction(this), '&Rename...');
            
            
           % Process Menu Definition 
            
            processMenu = uimenu(hf, 'Label', '&Process');

            addMenuItem(processMenu, AddShapeBoundingBoxAction(this), '&Bounding Box');
            addMenuItem(processMenu, AddShapeOrientedBoxAction(this), '&Oriented Box');
            addMenuItem(processMenu, AddShapeConvexHullAction(this), '&Convex Hull');
            addMenuItem(processMenu, AddShapeInertiaEllipseAction(this), 'Inertia &Ellipse');
            
            addMenuItem(processMenu, SimplifyPolygonAction(this), ...
                'Simplify Polygon/Polyline', true);
            
            addMenuItem(processMenu, ScaleShapeAction(this), '&Scale', true);
            addMenuItem(processMenu, TranslateShapeAction(this), '&Translate');

            
            % Tools Menu Definition 
            
            toolsMenu = uimenu(hf, 'Label', '&Tools');
            addMenuItem(toolsMenu, ...
                SelectToolAction(this, CreatePolygonTool(this)), ...
                'Create &Polygon');
            addMenuItem(toolsMenu, ...
                SelectToolAction(this, SelectionTool(this)), ...
                'Selection', true);
            
        end

        function item = addMenuItem(menu, action, label, varargin)
            
            % creates new item
            item = uimenu(menu, 'Label', label, ...
                'Callback', @action.actionPerformed);
            
            % eventually add separator above item
            if ~isempty(varargin)
                var = varargin{1};
                if islogical(var)
                    set(item, 'Separator', 'On');
                end
            end
        end
        
        function setupLayout(hf)
            
            % compute background color of most widgets
            bgColor = get(0, 'defaultUicontrolBackgroundColor');
            if ispc
                bgColor = 'White';
            end
            set(hf, 'defaultUicontrolBackgroundColor', bgColor);
            
            % vertical layout for putting status bar on bottom
            mainPanel = uiextras.VBox('Parent', hf, ...
                'Units', 'normalized', ...
                'Position', [0 0 1 1]);
            
            % horizontal panel: main view nmiddle, options left and right
            horzPanel = uiextras.HBoxFlex('Parent', mainPanel);
            
            % panel for doc info
            docInfoPanel = uiextras.VBoxFlex('Parent', horzPanel);

            % create a default uittree
            treePanel = uipanel(...
                'Parent', docInfoPanel, ...
                'Position', [0 0 1 1], ...
                'BorderType', 'none', ...
                'BorderWidth', 0);
            
            this.handles.shapeList = uicontrol(...
                'Style', 'listbox', ...
                'Parent', treePanel, ...
                'String', {'Circle', 'Poly1', 'Poly2', 'Ellipse'}, ...
                'Min', 1, 'Max', 4, ...
                'Units', 'normalized', ...
                'Position', [0 0 1 1], ...
                'Callback', @this.onShapeListModified);

%             displayOptionsPanel = uipanel(...
%                 'parent', docInfoPanel, ...
%                 'Position', [0 0 1 1], ...
%                 'BorderType', 'none', ...
%                 'BorderWidth', 0);
            displayOptionsPanel = uitable(...
                'parent', docInfoPanel, ...
                'Position', [0 0 1 1] );
            
                        
            docInfoPanel.Sizes = [-1 -1];
            
            this.handles.docInfoPanel = docInfoPanel;
            this.handles.displayOptionsPanel = displayOptionsPanel;
            


            % panel for image display
            displayPanel = uiextras.VBox('Parent', horzPanel);
            
            ax = axes('parent', displayPanel, ...
                'units', 'normalized', ...
                'dataAspectRatio', [1 1 1], ...
                'position', [0 0 1 1], ...
            	'XTick', [], ...
            	'YTick', [], ...
            	'Box', 'off');
            
            if ~isempty(this.doc.viewBox)
                set(ax, 'XLim', doc.viewBox(1:2));
                set(ax, 'YLim', doc.viewBox(3:4));
            end
            
            % keep widgets handles
            this.handles.mainAxis = ax;
            
            horzPanel.Sizes = [180 -1];
            
            % info panel for cursor position and value
            this.handles.statusBar = uicontrol(...
                'Parent', mainPanel, ...
                'Style', 'text', ...
                'String', ' x=    y=     I=', ...
                'HorizontalAlignment', 'left');
            
            % set up relative sizes of layouts
            mainPanel.Sizes = [-1 20];
        end
      
    end
    
end


%% Management of shape list

methods
    function shape = addShape(this, shape, varargin)
        % Add a shape to the doc and update the view
        %
        % addShape(SV, SHAPE)
        % Adds the specified shape to the doc and update the view.
        %
        % It is also possible to add a shape specified by a type and data
        % SHAPE = addShape(SV, TYPE, DATA)
        %
        
        if ~isstruct(shape) && ~isa(shape, 'svui.app.Shape');
            % create the new shape, defined by type, data, and style
            if nargin > 2
                style = varargin(2:end);
                shape = svui.app.Shape(shape, varargin{1}, style);
            else
                shape = svui.app.Shape(shape, varargin{1});
            end
        end
        
        % add shape to the doc and update view
        addShape(this.doc, shape);

        % update the various document views in this frame
        updateDisplay(this);
        updateShapeList(this);
    end
    
    function removeShape(this, shape)
        % removes the specified shape from the document, and update views
        
        % remove from doc
        removeShape(this.doc, shape);
        
        % update the various document views in this frame
        updateDisplay(this);
        updateShapeList(this);
    end
    
end


%% Management of selected shapes
methods
    function clearSelection(this)
        % remove all shapes in the selectedShapes field
        this.selectedShapes = [];
    end
    
    function addToSelection(this, shape)
        this.selectedShapes = [this.selectedShapes shape];
    end
    
    function removeFromSelection(this, shape)
        ind = find(shape == this.selectedShapes);
        if isempty(ind)
            warning('ShapeViewer:MainFrame:Selection', ...
                'could not find a shape in selection list');
            return;
        end
        this.selectedShapes(ind(1)) = [];
    end
    
    function onSelectionUpdated(this)
        updateShapeSelectionDisplay(this);
        updateShapeList(this);
    end
end


%% General methods

methods
    
    function updateDisplay(this)
        % refresh document display: clear axis, draw each shape, udpate axis
        
%         disp('update Display');
        
        % clear axis
        ax = this.handles.mainAxis;
        cla(ax);
        hold on;
        
        % initialize line handles for axis lines
        if this.doc.axisLinesVisible
            hl1 = plot([0 1], [0 0], 'k-');
            hl2 = plot([0 0], [0 1], 'k-');
        end
        
        % draw each shape in the document
        tool = this.currentTool;
        shapes = this.doc.shapes;
        for i = 1:length(shapes)
            shape = shapes(i);
            hs = draw(shape);
            set(hs, 'buttonDownFcn', @tool.onMouseClicked);
            set(hs, 'UserData', shape);
            
            if any(shape == this.selectedShapes)
                set(hs, 'Selected', 'on');
            end
        end
        
        % set axis bounds from view box
        if ~isempty(this.doc.viewBox)
            set(ax, 'XLim', this.doc.viewBox(1:2));
            set(ax, 'YLim', this.doc.viewBox(3:4));
        end
            
        % draw lines for X and Y axes, based on current axis bounds
        if this.doc.axisLinesVisible
            viewBox = this.doc.viewBox;
            if isempty(viewBox)
                viewBox = [get(ax, 'xlim') get(ax, 'ylim')];
            end
            set(hl1, 'XData', [viewBox(1) viewBox(2)], 'Ydata', [0 0]);
            set(hl2, 'Xdata', [0 0], 'YData', [viewBox(3) viewBox(4)]);
        end

        updateShapeList(this);
        
%         disp('end of update Display');
    end
    
    function updateShapeSelectionDisplay(this)
        % update the selected state of each shape
        
        % extract the list of handles in current axis
        ax = this.handles.mainAxis;
        children = get(ax, 'Children');
        
        % iterate over children
        for i = 1:length(children)
            % Extract shape referenced by current handle, if any
            shape = get(children(i), 'UserData');
            if isempty(shape) || ~isa(shape, 'svui.app.Shape')
                continue;
            end
            
            % update selection state of current shape
            if any(shape == this.selectedShapes)
                set(children(i), 'Selected', 'on');
            else
                set(children(i), 'Selected', 'off');
            end
        end
        
    end
    
    function updateTitle(this)
        % set up title of the figure, containing name of doc
        title = sprintf('%s - ShapeViewer', this.doc.name);
        set(this.handles.figure, 'Name', title);
    end
    
    
    function updateShapeList(this)
        % Refresh the shape tree when a shape is added or removed

%         disp('update shape list');
        
        nShapes = length(this.doc.shapes);
        shapeNames = cell(nShapes, 1);
        inds = [];
        for i = 1:nShapes
            shape = this.doc.shapes(i);
            
            % create name for current shape
            name = shape.name;
            if isempty(shape.name)
                name = ['(' class(shape.geometry) ')'];
            end
            shapeNames{i} = name;
            
            % create the set of selected indices
            if any(shape == this.selectedShapes)
                inds = [inds i]; %#ok<AGROW>
            end
        end

        % avoid empty indices, causing problems to gui...
        if nShapes > 0 && isempty(inds)
            inds = 1;
        end
        
        set(this.handles.shapeList, ...
            'String', shapeNames, ...
            'Max', nShapes, ...
            'Value', inds);
    end
end


%% Mouse listeners management
methods
    function addMouseListener(this, listener)
        % Add a mouse listener to this viewer
        this.mouseListeners = [this.mouseListeners {listener}];
    end
    
    function removeMouseListener(this, listener)
        % Remove a mouse listener from this viewer
        
        % find which listeners are the same as the given one
        inds = false(size(this.mouseListeners));
        for i = 1:numel(this.mouseListeners)
            if this.mouseListeners{i} == listener
                inds(i) = true;
            end
        end
        
        % remove first existing listener
        inds = find(inds);
        if ~isempty(inds)
            this.mouseListeners(inds(1)) = [];
        end
    end
    
    function processMouseButtonPressed(this, hObject, eventdata)
        % propagates mouse event to all listeners
        for i = 1:length(this.mouseListeners)
            onMouseButtonPressed(this.mouseListeners{i}, hObject, eventdata);
        end
    end
    
    function processMouseButtonReleased(this, hObject, eventdata)
        % propagates mouse event to all listeners
        for i = 1:length(this.mouseListeners)
            onMouseButtonReleased(this.mouseListeners{i}, hObject, eventdata);
        end
    end
    
    function processMouseMoved(this, hObject, eventdata)
        % propagates mouse event to all listeners
        for i = 1:length(this.mouseListeners)
            onMouseMoved(this.mouseListeners{i}, hObject, eventdata);
        end
    end
end


%% Widget callbacks

methods
    function onShapeListModified(this, varargin)
        
%         disp('shape list updated');
        
        inds = get(this.handles.shapeList, 'Value');
        if isempty(inds)
            return;
        end
        
        this.selectedShapes = this.doc.shapes(inds);
        updateShapeSelectionDisplay(this);
    end
end

%% Figure management
methods
    function close(this, varargin)
%         disp('Close shape viewer frame');
        delete(this.handles.figure);
    end
    
    function onFigureResized(this, varargin)
        updateShapeSelectionDisplay(this);
    end
end

end